@@ -4,7 +4,7 @@ import Foundation
 
 /// Main manager that coordinates multitouch monitoring and gesture recognition
 /// Thread-safety: Uses internal gestureQueue for synchronization of touch processing
-final class MultitouchManager: @unchecked Sendable {
+public final class MultitouchManager: @unchecked Sendable {
 
     // MARK: - Constants
 
@@ -33,7 +33,7 @@ final class MultitouchManager: @unchecked Sendable {
     private(set) var isEnabled = false
 
     /// Whether monitoring is active
-    private(set) var isMonitoring = false
+    public private(set) var isMonitoring = false
 
     /// Whether currently in a three-finger gesture (used for event suppression)
     private(set) var isInThreeFingerGesture = false
@@ -118,7 +118,7 @@ final class MultitouchManager: @unchecked Sendable {
 
     /// Shared production instance
     /// Note: Initialized once at app startup, accessed from main thread and gesture queue
-    static let shared = MultitouchManager()
+    public static let shared = MultitouchManager()
 
     /// Initialize with optional factories for dependency injection
     /// - Parameters:
@@ -148,7 +148,7 @@ final class MultitouchManager: @unchecked Sendable {
     // MARK: - Public Interface
 
     /// Start monitoring for gestures
-    func start() {
+    public func start() {
         guard !isMonitoring else { return }
 
         applyConfiguration()
@@ -181,7 +181,7 @@ final class MultitouchManager: @unchecked Sendable {
     }
 
     /// Stop monitoring
-    func stop() {
+    public func stop() {
         // Clear restart state and cancel any pending restart work item.
         // This must be done under lock to prevent data races with restart().
         restartLock.lock()
@@ -202,7 +202,7 @@ final class MultitouchManager: @unchecked Sendable {
     }
 
     /// Restart monitoring (used after sleep/wake)
-    func restart() {
+    public func restart() {
         // Allow restart if either:
         // 1. wakeObserver exists (normal production case after successful start)
         // 2. isMonitoring is true (for test scenarios where event tap setup may fail)
@@ -277,7 +277,7 @@ final class MultitouchManager: @unchecked Sendable {
     }
 
     /// Performs the actual restart after the cleanup delay
-    private func performRestart(wasEnabled: Bool) {
+    func performRestart(wasEnabled: Bool) {
         // Verify we should still restart (manager may have been stopped during delay)
         guard wakeObserver != nil else {
             markRestartComplete()
@@ -397,7 +397,7 @@ final class MultitouchManager: @unchecked Sendable {
     }
 
     /// Update configuration
-    func updateConfiguration(_ config: GestureConfiguration) {
+    public func updateConfiguration(_ config: GestureConfiguration) {
         configuration = config
         applyConfiguration()
     }
@@ -652,42 +652,33 @@ extension MultitouchManager: DeviceMonitorDelegate {
         // CGEventSource.flagsState is thread-safe and can be called from any thread
         let modifierFlags = CGEventSource.flagsState(.hidSystemState)
 
-        // CRITICAL: Copy touch data synchronously during this callback.
         // The touches pointer is only valid for the duration of this callback.
-        // The MultitouchSupport framework owns this memory and may free/reuse it
-        // immediately after the callback returns. We MUST copy the data before
-        // dispatching to the gesture queue.
+        // Copy touch data via raw memcpy — much cheaper than Swift Array allocation
+        // + map closure that was causing per-frame GC pressure and jitter at 100Hz+.
         let touchCount = Int(count)
-        let touchDataCopy: [MTTouch]
+        nonisolated(unsafe) let touchesPtr: UnsafeMutableRawPointer?
         if touchCount > 0 {
-            let touchArray = unsafe touches.bindMemory(to: MTTouch.self, capacity: touchCount)
-            // Create a Swift array copy of the touch data
-            touchDataCopy = (0..<touchCount).map { unsafe touchArray[$0] }
+            let byteCount = touchCount * MemoryLayout<MTTouch>.stride
+            let buffer = UnsafeMutableRawPointer.allocate(
+                byteCount: byteCount, alignment: MemoryLayout<MTTouch>.alignment)
+            unsafe buffer.copyMemory(from: touches, byteCount: byteCount)
+            unsafe touchesPtr = unsafe buffer
         } else {
-            touchDataCopy = []
+            unsafe touchesPtr = nil
         }
 
-        // Gesture recognition and finger counting is done inside processTouches
-        // State updates happen in delegate callbacks dispatched to main thread
-        // IMPORTANT: We must call processTouches even with zero touches so the
-        // gesture recognizer can properly end gestures via stableFrameCount logic.
         gestureQueue.async { [weak self] in
-            if touchDataCopy.isEmpty {
-                // Zero touches - still need to notify gesture recognizer so it can
-                // properly end active gestures via stableFrameCount mechanism
+            if let buffer = unsafe touchesPtr {
+                defer { unsafe buffer.deallocate() }
                 unsafe self?.gestureRecognizer.processTouches(
-                    UnsafeMutableRawPointer(bitPattern: 1)!, // Non-null placeholder, won't be dereferenced when count=0
+                    buffer, count: touchCount, timestamp: timestamp, modifierFlags: modifierFlags)
+            } else {
+                // Zero touches — still notify so gesture recognizer can end via stableFrameCount
+                unsafe self?.gestureRecognizer.processTouches(
+                    UnsafeMutableRawPointer(bitPattern: 1)!,
                     count: 0,
                     timestamp: timestamp,
                     modifierFlags: modifierFlags)
-            } else {
-                // Use withUnsafeBufferPointer to get a pointer to our copied data
-                unsafe touchDataCopy.withUnsafeBufferPointer { buffer in
-                    guard let baseAddress = buffer.baseAddress else { return }
-                    let rawPointer = unsafe UnsafeMutableRawPointer(mutating: baseAddress)
-                    unsafe self?.gestureRecognizer.processTouches(
-                        rawPointer, count: touchCount, timestamp: timestamp, modifierFlags: modifierFlags)
-                }
             }
         }
     }
