@@ -1,9 +1,15 @@
 import XCTest
 
-@testable import MiddleDrag
+@testable import MiddleDragCore
 
 final class MultitouchManagerTests: XCTestCase {
 
+    private func requireCGEventTestsEnabled() throws {
+        if ProcessInfo.processInfo.environment["RUN_CGEVENT_TESTS"] == "0" {
+            throw XCTSkip("Skipping CGEvent-based tests; set RUN_CGEVENT_TESTS=1 to run.")
+        }
+    }
+
     // MARK: - Singleton Tests
 
     func testSharedInstanceIsSingleton() {
@@ -1466,7 +1472,8 @@ final class MultitouchManagerTests: XCTestCase {
 
     // MARK: - Event Processing Logic Tests
 
-    func testProcessEventPassesThroughTapDisabledEvents() {
+    func testProcessEventPassesThroughTapDisabledEvents() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1478,7 +1485,8 @@ final class MultitouchManagerTests: XCTestCase {
         unsafe XCTAssertNotNil(result)
     }
 
-    func testProcessEventIdentifiesOurEvents() {
+    func testProcessEventIdentifiesOurEvents() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1500,7 +1508,8 @@ final class MultitouchManagerTests: XCTestCase {
         }
     }
 
-    func testProcessEventIntercptsForceClick() {
+    func testProcessEventIntercptsForceClick() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1520,7 +1529,8 @@ final class MultitouchManagerTests: XCTestCase {
             result, "Physical left click with 3 fingers should be suppressed (Force Click)")
     }
 
-    func testProcessEventPassesNormalLeftClick() {
+    func testProcessEventPassesNormalLeftClick() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1537,7 +1547,8 @@ final class MultitouchManagerTests: XCTestCase {
         unsafe XCTAssertNotNil(result, "Normal left click should pass through")
     }
 
-    func testProcessEventSuppressesDuringGesture() {
+    func testProcessEventSuppressesDuringGesture() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1569,7 +1580,8 @@ final class MultitouchManagerTests: XCTestCase {
     // cannot be easily mocked in unit tests. The modifier key logic is tested through
     // integration testing and manual verification.
 
-    func testProcessEventSuppressesWhenModifierHeldAndGestureActive() {
+    func testProcessEventSuppressesWhenModifierHeldAndGestureActive() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1608,7 +1620,8 @@ final class MultitouchManagerTests: XCTestCase {
 
     // MARK: - Last Gesture Was Active Flag Tests
 
-    func testCancelledGestureDoesNotSuppressEventsAfterEnd() {
+    func testCancelledGestureDoesNotSuppressEventsAfterEnd() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1651,7 +1664,8 @@ final class MultitouchManagerTests: XCTestCase {
         manager.stop()
     }
 
-    func testActiveGestureSuppressesEventsAfterEnd() {
+    func testActiveGestureSuppressesEventsAfterEnd() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1699,7 +1713,8 @@ final class MultitouchManagerTests: XCTestCase {
         manager.stop()
     }
 
-    func testCancelledDragDoesNotSuppressEventsAfterEnd() {
+    func testCancelledDragDoesNotSuppressEventsAfterEnd() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1748,7 +1763,8 @@ final class MultitouchManagerTests: XCTestCase {
         manager.stop()
     }
 
-    func testActiveDragSuppressesEventsAfterEnd() {
+    func testActiveDragSuppressesEventsAfterEnd() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1798,7 +1814,8 @@ final class MultitouchManagerTests: XCTestCase {
         manager.stop()
     }
 
-    func testEventSuppressionWindowExpires() {
+    func testEventSuppressionWindowExpires() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -1843,7 +1860,8 @@ final class MultitouchManagerTests: XCTestCase {
     }
 
 
-    func testToggleEnabledResetsLastGestureWasActive() {
+    func testToggleEnabledResetsLastGestureWasActive() throws {
+        try requireCGEventTestsEnabled()
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
@@ -2192,12 +2210,12 @@ final class MultitouchManagerTests: XCTestCase {
         manager.stop()
     }
 
-    func testMainActorIsolatedCallsDoNotDeadlock() {
+    func testMainActorIsolatedCallsDoNotDeadlock() throws {
         // Stress test to ensure no deadlock from rapid thread switches
         let mockDevice = unsafe MockDeviceMonitor()
         let manager = MultitouchManager(
             deviceProviderFactory: { unsafe mockDevice }, eventTapSetup: { true })
-        let recognizer = GestureRecognizer()
+        nonisolated(unsafe) let recognizer = GestureRecognizer()
 
         var config = GestureConfiguration()
         config.middleDragEnabled = true
@@ -2216,16 +2234,16 @@ final class MultitouchManagerTests: XCTestCase {
         for _ in 0..<iterations {
             // Main thread call
             DispatchQueue.main.async {
-                manager.gestureRecognizerDidStart(recognizer, at: MTPoint(x: 0.5, y: 0.5))
-                manager.gestureRecognizerDidBeginDragging(recognizer)
-                manager.gestureRecognizerDidEndDragging(recognizer)
+                unsafe manager.gestureRecognizerDidStart(recognizer, at: MTPoint(x: 0.5, y: 0.5))
+                unsafe manager.gestureRecognizerDidBeginDragging(recognizer)
+                unsafe manager.gestureRecognizerDidEndDragging(recognizer)
                 expectation.fulfill()
             }
 
             // Background thread call
             DispatchQueue.global(qos: .userInitiated).async {
-                manager.gestureRecognizerDidStart(recognizer, at: MTPoint(x: 0.5, y: 0.5))
-                manager.gestureRecognizerDidTap(recognizer)
+                unsafe manager.gestureRecognizerDidStart(recognizer, at: MTPoint(x: 0.5, y: 0.5))
+                unsafe manager.gestureRecognizerDidTap(recognizer)
                 expectation.fulfill()
             }
         }
